@@ -1,8 +1,15 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 
 use crate::colors;
+use crate::file_fetcher::get_source_from_bytes;
+use crate::file_fetcher::strip_shebang;
+use crate::flags::Flags;
+use crate::ops;
+use crate::program_state::ProgramState;
 use crate::version;
+use data_url::DataUrl;
 use deno_core::error::type_error;
+use deno_core::error::uri_error;
 use deno_core::error::AnyError;
 use deno_core::error::Context;
 use deno_core::futures::FutureExt;
@@ -15,7 +22,8 @@ use deno_core::v8_set_flags;
 use deno_core::ModuleLoader;
 use deno_core::ModuleSpecifier;
 use deno_core::OpState;
-use deno_runtime::deno_file::BlobUrlStore;
+use deno_runtime::deno_broadcast_channel::InMemoryBroadcastChannel;
+use deno_runtime::deno_web::BlobUrlStore;
 use deno_runtime::permissions::Permissions;
 use deno_runtime::permissions::PermissionsOptions;
 use deno_runtime::worker::MainWorker;
@@ -108,6 +116,19 @@ fn read_string_slice(
   Ok(string)
 }
 
+fn get_source_from_data_url(
+  specifier: &ModuleSpecifier,
+) -> Result<String, AnyError> {
+  let data_url = DataUrl::process(specifier.as_str())
+    .map_err(|e| uri_error(format!("{:?}", e)))?;
+  let mime = data_url.mime_type();
+  let charset = mime.get_parameter("charset").map(|v| v.to_string());
+  let (bytes, _) = data_url
+    .decode_to_vec()
+    .map_err(|e| uri_error(format!("{:?}", e)))?;
+  Ok(strip_shebang(get_source_from_bytes(bytes, charset)?))
+}
+
 const SPECIFIER: &str = "file://$deno$/bundle.js";
 
 struct EmbeddedModuleLoader(String);
@@ -120,12 +141,16 @@ impl ModuleLoader for EmbeddedModuleLoader {
     _referrer: &str,
     _is_main: bool,
   ) -> Result<ModuleSpecifier, AnyError> {
-    if specifier != SPECIFIER {
-      return Err(type_error(
-        "Self-contained binaries don't support module loading",
-      ));
+    if let Ok(module_specifier) = resolve_url(&specifier) {
+      if get_source_from_data_url(&module_specifier).is_ok()
+        || specifier == SPECIFIER
+      {
+        return Ok(module_specifier);
+      }
     }
-    Ok(resolve_url(specifier)?)
+    Err(type_error(
+      "Self-contained binaries don't support module loading",
+    ))
   }
 
   fn load(
@@ -136,13 +161,19 @@ impl ModuleLoader for EmbeddedModuleLoader {
     _is_dynamic: bool,
   ) -> Pin<Box<deno_core::ModuleSourceFuture>> {
     let module_specifier = module_specifier.clone();
-    let code = self.0.to_string();
+    let is_data_uri = get_source_from_data_url(&module_specifier).ok();
+    let code = if let Some(ref source) = is_data_uri {
+      source.to_string()
+    } else {
+      self.0.to_string()
+    };
     async move {
-      if module_specifier.to_string() != SPECIFIER {
+      if is_data_uri.is_none() && module_specifier.to_string() != SPECIFIER {
         return Err(type_error(
           "Self-contained binaries don't support module loading",
         ));
       }
+
       Ok(deno_core::ModuleSource {
         code,
         module_url_specified: module_specifier.to_string(),
@@ -153,13 +184,36 @@ impl ModuleLoader for EmbeddedModuleLoader {
   }
 }
 
+fn metadata_to_flags(metadata: &Metadata) -> Flags {
+  let permissions = metadata.permissions.clone();
+  Flags {
+    argv: metadata.argv.clone(),
+    unstable: metadata.unstable,
+    seed: metadata.seed,
+    location: metadata.location.clone(),
+    allow_env: permissions.allow_env,
+    allow_hrtime: permissions.allow_hrtime,
+    allow_net: permissions.allow_net,
+    allow_plugin: permissions.allow_plugin,
+    allow_read: permissions.allow_read,
+    allow_run: permissions.allow_run,
+    allow_write: permissions.allow_write,
+    v8_flags: metadata.v8_flags.clone(),
+    log_level: metadata.log_level,
+    ..Default::default()
+  }
+}
+
 pub async fn run(
   source_code: String,
   metadata: Metadata,
 ) -> Result<(), AnyError> {
+  let flags = metadata_to_flags(&metadata);
   let main_module = resolve_url(SPECIFIER)?;
+  let program_state = ProgramState::build(flags).await?;
   let permissions = Permissions::from_options(&metadata.permissions);
   let blob_url_store = BlobUrlStore::default();
+  let broadcast_channel = InMemoryBroadcastChannel::default();
   let module_loader = Rc::new(EmbeddedModuleLoader(source_code));
   let create_web_worker_cb = Arc::new(|_| {
     todo!("Worker are currently not supported in standalone binaries");
@@ -191,15 +245,26 @@ pub async fn run(
     no_color: !colors::use_color(),
     get_error_class_fn: Some(&get_error_class_name),
     location: metadata.location,
-    location_data_dir: None,
+    origin_storage_dir: None,
     blob_url_store,
+    broadcast_channel,
   };
   let mut worker =
     MainWorker::from_options(main_module.clone(), permissions, &options);
+  {
+    let js_runtime = &mut worker.js_runtime;
+    js_runtime
+      .op_state()
+      .borrow_mut()
+      .put::<Arc<ProgramState>>(program_state.clone());
+    ops::errors::init(js_runtime);
+    ops::runtime_compiler::init(js_runtime);
+    js_runtime.sync_ops_cache();
+  }
   worker.bootstrap(&options);
   worker.execute_module(&main_module).await?;
   worker.execute("window.dispatchEvent(new Event('load'))")?;
-  worker.run_event_loop().await?;
+  worker.run_event_loop(true).await?;
   worker.execute("window.dispatchEvent(new Event('unload'))")?;
   std::process::exit(0);
 }
