@@ -72,6 +72,9 @@ delete Object.prototype.__proto__;
   /** @type {Map<string, ts.SourceFile>} */
   const sourceFileCache = new Map();
 
+  /** @type {Map<string, string>} */
+  const scriptVersionCache = new Map();
+
   /** @param {ts.DiagnosticRelatedInformation} diagnostic */
   function fromRelatedInformation({
     start,
@@ -146,6 +149,9 @@ delete Object.prototype.__proto__;
     // TS2306: File 'file:///Users/rld/src/deno/cli/tests/subdir/amd_like.js' is
     // not a module.
     2306,
+    // TS2688: Cannot find type definition file for '...'.
+    // We ignore because type defintion files can end with '.ts'.
+    2688,
     // TS2691: An import path cannot end with a '.ts' extension. Consider
     // importing 'bad-module' instead.
     2691,
@@ -368,7 +374,15 @@ delete Object.prototype.__proto__;
       if (sourceFile) {
         return sourceFile.version ?? "1";
       }
-      return core.opSync("op_script_version", { specifier });
+      // tsc neurotically requests the script version multiple times even though
+      // it can't possibly have changed, so we will memoize it on a per request
+      // basis.
+      if (scriptVersionCache.has(specifier)) {
+        return scriptVersionCache.get(specifier);
+      }
+      const scriptVersion = core.opSync("op_script_version", { specifier });
+      scriptVersionCache.set(specifier, scriptVersion);
+      return scriptVersion;
     },
     getScriptSnapshot(specifier) {
       debug(`host.getScriptSnapshot("${specifier}")`);
@@ -386,8 +400,7 @@ delete Object.prototype.__proto__;
           },
         };
       }
-      /** @type {string | undefined} */
-      const version = core.opSync("op_script_version", { specifier });
+      const version = host.getScriptVersion(specifier);
       if (version != null) {
         return new ScriptSnapshot(specifier, version);
       }
@@ -526,6 +539,8 @@ delete Object.prototype.__proto__;
    */
   function serverRequest({ id, ...request }) {
     debug(`serverRequest()`, { id, ...request });
+    // evict all memoized source file versions
+    scriptVersionCache.clear();
     switch (request.method) {
       case "configure": {
         const { options, errors } = ts
